@@ -1,7 +1,8 @@
 package Plack::Middleware::TemplateToolkit;
 BEGIN {
-  $Plack::Middleware::TemplateToolkit::VERSION = '0.15';
+  $Plack::Middleware::TemplateToolkit::VERSION = '0.2';
 }
+# ABSTRACT: Serve files with Template Toolkit and Plack
 
 use strict;
 use warnings;
@@ -60,8 +61,9 @@ BEGIN {
 }
 
 use Plack::Util::Accessor (
-    qw(dir_index path extension content_type default_type
-        tt pass_through utf8_downgrade utf8_allow vars root), @TT_CONFIG
+    qw(dir_index path extension content_type default_type tt root
+        pass_through decode_request encode_response vars request_vars),
+    @TT_CONFIG
 );
 
 sub prepare_app {
@@ -70,16 +72,17 @@ sub prepare_app {
     $self->dir_index('index.html')   unless $self->dir_index;
     $self->pass_through(0)           unless defined $self->pass_through;
     $self->default_type('text/html') unless $self->default_type;
+    $self->decode_request('utf8')    unless defined $self->decode_request;
+    $self->encode_response('utf8')   unless defined $self->encode_response;
+    $self->request_vars( [] ) unless defined $self->request_vars;
 
-    if ( not ref $self->vars ) {
-        $self->vars(
-            sub {
-                { shift->query_parameters }
-            }
-        );
-    } elsif ( ref $self->vars ne 'CODE' ) {
+    if ( not $self->vars ) {
+        $self->vars( sub { return { params => shift->query_parameters } } );
+    } elsif ( ref $self->vars eq 'HASH' ) {
         my $vars = $self->vars;
-        $self->vars( sub {$vars} );
+        $self->vars( sub { return $vars; } );
+    } elsif ( ref $self->vars ne 'CODE' ) {
+        die 'vars must be a code or hash reference, if defined';
     }
 
     my $config = {};
@@ -100,7 +103,7 @@ sub prepare_app {
     }
 }
 
-sub call {    # adopted from Plack::Middleware::Static
+sub call {    # adapted from Plack::Middleware::Static
     my ( $self, $env ) = @_;
 
     my $res = $self->_handle_template($env);
@@ -111,7 +114,7 @@ sub call {    # adopted from Plack::Middleware::Static
     if ( $self->app ) {
         $res = $self->app->($env);
 
-        # TODO: if $res->[0] ne 200 and catch_errors: process error message
+        # TODO: if $res->[0] ne 200 and catch_errors: process error message?
     } else {
         my $req = Plack::Request->new($env);
         $res = $self->process_error( 404, 'Not found', 'text/plain', $req );
@@ -123,23 +126,21 @@ sub call {    # adopted from Plack::Middleware::Static
 sub process_template {
     my ( $self, $template, $code, $vars ) = @_;
 
-    my $content;
+    my ( $content, $res );
     if ( $self->tt->process( $template, $vars, \$content ) ) {
         my $type = $self->content_type || do {
             Plack::MIME->mime_type($1) if $template =~ /(\.\w{1,6})$/;
             }
             || $self->default_type;
-        if ( not $self->utf8_allow ) {
-            $content = encode_utf8($content);
-        } elsif ( $self->utf8_downgrade ) {
-
-            # this undocumented option does not fix but makes errors visible
-            utf8::downgrade($content);
+        if ( $self->encode_response ) {
+            $content = encode( $self->encode_response, $content );
         }
-        return [ $code, [ 'Content-Type' => $type ], [$content] ];
+        $res = [ $code, [ 'Content-Type' => $type ], [$content] ];
     } else {
-        return $self->tt->error->as_string;
+        $res = $self->tt->error->as_string;
     }
+
+    return $res;
 }
 
 sub process_error {
@@ -159,41 +160,81 @@ sub process_error {
     $self->_set_vars($req);
 
     $req->env->{'tt.vars'}->{'error'} = $error;
-    my $res = $self->process_template( $self->{$code}, $code,
-        $req->env->{'tt.vars'} );
+    my $tpl = $self->{$code};
+    my $res = $self->process_template( $tpl, $code, $req->env->{'tt.vars'} );
 
-    if ( not ref $res ) {
+    unless ( ref $res ) {
 
         # processing error document failed: result in a 500 error
         if ( $code eq 500 ) {
             $res = [ 500, [ 'Content-Type' => $type ], [$res] ];
+            $tpl = undef;
         } else {
             if ( ref $req->logger ) {
                 $req->logger->( { level => 'warn', message => $res } );
             }
-            $res = $self->process_error( 500, $res, $type, $req );
+            ( $res, $tpl ) = $self->process_error( 500, $res, $type, $req );
         }
     }
 
-    return $res;
+    return wantarray ? ( $res, $tpl ) : $res;
 }
 
 sub _set_vars {
     my ( $self, $req ) = @_;
     my $env = $req->env;
 
-    # TODO: $self->vars may die if it's broken. Should be catch this?
-    my $vars = $self->vars->($req) if defined $self->{vars};
+    # we must not copy the vars by reference because
+    # otherwise we might modify the same object
+    # TODO: catch error if $self->vars does not return hash ref or dies?
+    my (%vars) = %{ $self->vars->($req) } if defined $self->vars;
+
+    my $rv = $self->request_vars;
+    unless ( exists $vars{request} ) {
+        if ( $rv eq 'all' ) {
+            $vars{request} = $req;
+        } elsif ( ref $rv and @$rv ) {
+            $vars{request} = {};
+            foreach ( @{ $self->request_vars } ) {
+                next unless $req->can($_);
+                my $value = $req->$_;
+
+              # request vars should also be byte strings, so we must decode it
+                if ( $self->decode_request ) {
+                    my $encoding = $self->decode_request;
+
+                    if ( blessed($value) and $value->isa('Hash::MultiValue') )
+                    {
+                        my @values = $value->values;
+                        @values = map { decode( $encoding, $_ ) } @values;
+                        my $hash = Hash::MultiValue->new;
+                        foreach my $key ( $value->keys ) {
+                            $key = decode( $encoding, $key );
+                            $hash->add( $key, shift @values );
+                        }
+                        $value = $hash;
+                    } else {
+                        $value = decode( $encoding, $value );
+                    }
+                }
+
+                $vars{request}->{$_} = $value;
+            }
+        }
+    }
 
     if ( $env->{'tt.vars'} ) {
-        foreach ( keys %$vars ) {
-            $env->{'tt.vars'}->{$_} = $vars->{$_};
+
+        # add to existing vars
+        foreach ( keys %vars ) {
+            $env->{'tt.vars'}->{$_} = $vars{$_};
         }
     } else {
-        $env->{'tt.vars'} = $vars;
+        $env->{'tt.vars'} = \%vars;
     }
 }
 
+# core function called once in 'call'
 sub _handle_template {
     my ( $self, $env ) = @_;
 
@@ -210,64 +251,68 @@ sub _handle_template {
 
     my $req = Plack::Request->new($env);
 
+    $env->{'tt.path'} = $req->path_info;
+
     $path = $req->path;
     $path .= $self->dir_index if $path =~ /\/$/;
+    $path =~ s{^/}{};    # Do not want to enable absolute paths
 
     my $extension = $self->extension;
     if ( $extension and $path !~ /${extension}$/ ) {
 
         # TODO: we may want another code (forbidden) and message here
-        return $self->process_error( 404, 'Not found', 'text/plain', $req );
+        my ( $res, $tpl )
+            = $self->process_error( 404, 'Not found', 'text/plain', $req );
+        $env->{'tt.template'} = $tpl;
+        return $res;
     }
 
-    $path =~ s{^/}{};    # Do not want to enable absolute paths
-
     $self->_set_vars($req);
 
-    $env->{'tt.template'} = $path;    # for debug inspection (not tested)
-
+    my $tpl = $path;
     my $res = $self->process_template( $path, 200, $env->{'tt.vars'} );
-    if ( ref $res ) {
-        return $res;
-    } else {
+
+    unless ( ref $res ) {
         my $type = $self->content_type || $self->default_type;
         if ( $res =~ /file error .+ not found/ ) {
-            return $self->process_error( 404, $res, $type, $req );
+            ( $res, $tpl ) = $self->process_error( 404, $res, $type, $req );
         } else {
             if ( ref $req->logger ) {
                 $req->logger->( { level => 'warn', message => $res } );
             }
-            return $self->process_error( 500, $res, $type, $req );
+            ( $res, $tpl ) = $self->process_error( 500, $res, $type, $req );
         }
     }
+
+    $env->{'tt.template'} = $tpl;    # not fully covered by unit tests
+
+    return $res;
 }
 
 1;
 
 __END__
 
-=head1 NAME
-
-Plack::Middleware::TemplateToolkit - Serve files with Template Toolkit and Plack
-
 =head1 SYNOPSIS
 
     use Plack::Builder;
 
+    my $root = '/path/to/html_doc_root';
+
     builder {
 
         # Page to show when requested file is missing
-        enable "Plack::Middleware::ErrorDocument",
+        enable 'ErrorDocument',    #
             404 => "$root/page_not_found.html";
 
         # These files can be served directly
-        enable "Plack::Middleware::Static",
+        enable 'Static',
             path => qr{\.[gif|png|jpg|swf|ico|mov|mp3|pdf|js|css]$},
-            INCLUDE_PATH => $root;
+            root => $root;
 
-        enable "Plack::Middleware::TemplateToolkit",
-            INCLUDE_PATH => '/path/to/htdocs/', # required
-            pass_through => 1; # delegate missing templates to $app
+        enable 'TemplateToolkit',
+            INCLUDE_PATH => $root,    # required
+            pass_through => 1;        # delegate missing templates to $app
 
         $app;
     }
@@ -275,7 +320,6 @@ Plack::Middleware::TemplateToolkit - Serve files with Template Toolkit and Plack
 A minimal L<.psgi|PSGI> script as stand-alone application:
 
     use Plack::Middleware::TemplateToolkit;
-
     Plack::Middleware::TemplateToolkit->new( INCLUDE_PATH => "/path/to/docs" );
 
 =head1 DESCRIPTION
@@ -301,6 +345,7 @@ to migrate later so you might want to look at a propper framework such as
 L<Catalyst> if you do want to use them:
 
   [% params.get('field') %] params is a L<Hash::MultiValue>
+  [% request.parameters.field %] configured with request_vars => ['parameters']
 
 =head1 CONFIGURATIONS
 
@@ -342,6 +387,18 @@ variables. By default only the QUERY_STRING params are provided as 'params'.
 Templates variables specified by this option are added to existing template
 variables in the tt.vars environment variable.
 
+=item request_vars
+
+Specify a list of request variables from L<Plack::Request> to be collected
+in a template variable 'request'. For instance C<<[path base]>> gives you
+the template variables C<request.path> and C<request.base>. Setting this
+parameter to 'all' gives you the original Plack::Request object, but this
+is unstable, bad practice because the object may change and your templates
+may damage the request object.
+
+By default the request variables are decoded from byte strings to Unicode.
+You can change this with the configuration value 'decode_request'.
+
 =item dir_index
 
 Which file to use as a directory index, defaults to index.html
@@ -364,14 +421,24 @@ Directly set an instance of L<Template> instead of creating a new one:
   my $tt = Template->new( %tt_options );
   Plack::Middleware::TemplateToolkit->new( tt => $tt );
 
-=item utf8_allow
+=item encode_response
 
-PSGI expects the content body to be a byte stream, but Template Toolkit
-is best used with templates and variables as UTF8 strings. For this reason
-processed templates are encoded to UTF8 byte streams unless you enable this
-options. It is then up to you to ensure that only byte streams are emitted
-by your PSGI application. It is recommended to use L<Plack::Middleware::Lint>
-and test with Unicode characters or your application will likely fail.
+If your templates or template variables are Unicode strings, the output must be
+encoded, because PSGI expects the content body to be a byte stream. You can
+specify an encoding, such as 'utf8' with this parameter, so the output is
+encoded to a byte string. The default setting is 'utf8' which encodes to UTF-8
+bytes.  This default option is useful if your input contains non-ASCII
+characters, but it may lead to double encoded UTF-8 bytes, if you accidently
+mix strings with UTF-8 flag and without.  To find such implicit encoding
+conversions, try L<encoding::warnings>.
+
+=item decode_request
+
+Similar to 'encode_response', this parameter decodes the input request from a
+byte string to an encoding of your choice. Set to 'utf8' by default.
+
+It is highly recommended to use L<Plack::Middleware::Lint> and test your app
+with Unicode from several sources (templates, variables, parameters, ...).
 
 =back
 
@@ -385,6 +452,30 @@ In addition you can specify templates for error codes, for instance:
 If a specified error templates could not be found and processed, an error
 with HTTP status code 500 is returned, possibly also as template.
 
+=head1 ENVIRONMENT
+
+This middleware inspects and/or manipulates the following variables from 
+the PSGI environment:
+
+=over 4
+
+=item tt.vars
+
+Injected as template variables if defined. Set to the template variables.
+
+=item tt.path
+
+Set to the template that was asked to process. This is equal to the local path
+(C<path_info> in L<Plack::Request>) if the request matched.
+
+=item tt.template
+
+Set to the template that has actually been processed.
+
+=back
+
+You can view these variables with L<Plack::Middleware::Debug::TemplateToolkit>.
+
 =head1 METHODS
 
 In addition to the call() method derived from L<Plack::Middleware>, this
@@ -403,10 +494,12 @@ On failure this method returns an error message instead of a reference.
 Returns a PSGI response to be used as error message. Error templates are used
 if they have been specified and prepare_app has been called before. This method 
 tries hard not to fail: undefined parameters are replaced by default values.
+In list context this returns a PSGI response and the actual template that has
+been used to create the error document.
 
 =head1 SEE ALSO
 
-L<Plack>, L<Template>
+L<Plack>, L<Template>, L<Plack::Middleware::Debug::TemplateToolkit>
 
 =head1 AUTHORS
 
